"""Top-level package."""

__author__ = """Tyler Trimble, Meghna Sarah Thomas"""
__email__ = 'TylerL.Trimble@utexas.edu,meghnathomas@utexas.edu'
__version__ = '0.1.1'

import warnings

from viswaternet import utils
from viswaternet import network
from viswaternet.network.initialize import VisWNModel
from viswaternet import drawing
